#include <stdio.h>
#include <string.h>
#include "PlayTile.h"



PlayTile MakePlayTile(char* pName, Direction pMoving, bool pMoveable, bool pGivesDirections, 
                   bool pForceDirections, PointAt pPointDirection, DirType pMoveType, 
                   bool pAllowed[4], int pX, int pY, ColourType pColour, AlignType pAlignment,
                   bool pMurderer, bool pSuicidal, bool pLiveable, bool pInTheWay, 
                   bool pImportant, CollectType pCollect)
{
 PlayTile newTile;                   
 newTile.Name = pName;
 newTile.Moving = pMoving;
 newTile.Moveable = pMoveable;
 newTile.GivesDirections = pGivesDirections;
 newTile.ForceDirections = pForceDirections;
 newTile.PointDirection = pPointDirection;
 newTile.MoveType = pMoveType;
 newTile.Allowed[0] = pAllowed[0]; newTile.Allowed[1] = pAllowed[1]; 
 newTile.Allowed[2] = pAllowed[2]; newTile.Allowed[3] = pAllowed[3];
 newTile.X = pX;
 newTile.Y = pY;
 newTile.Colour = pColour;
 newTile.Alignment = pAlignment;
 newTile.Murderer = pMurderer;
 newTile.Suicidal = pSuicidal;
 newTile.Liveable = pLiveable;
 newTile.InTheWay = pInTheWay;
 newTile.Important = pImportant;
 newTile.Collect = pCollect;
 newTile.RemoveMe = false;
 newTile.Dirty = false;
 newTile.NewMoving = PNONE;
 newTile.Collected = false;
 
 return newTile;
}

PlayTile MakeStandardTile(char* pName, ColourType pColour, bool pMoveable, PointAt pPointDirection, int X, int Y)
//This function simplifies the tile creating process.  This function will create a tile based on
//parameters given.  However, since this creates only the standard tiles as specified by the
//first documentation, parameters you give may be ignored based on the tile's name.
{
  bool Allowed[] = {true,true,true,true};
  if(strcmp("PID", pName) == 0) 
    return MakePlayTile(pName, STOPPED, pMoveable, true, false, PNONE, NORMAL, Allowed, X, Y,
                        ISGREY, HERO, false, true, true, true, true, COLLECTOR);
  else if(strcmp("CRYSTAL", pName) == 0)
    return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, false, false, false, true, true, COLLECTABLE);
  else if(strcmp("CLOSEDEXIT", pName) == 0) 
    return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, false, false, false, false, true, COLLECTWAIT);  
  else if(strcmp("ALLY", pName) == 0) 
    return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        ISGREY, HERO, false, true, true, true, true, COLLECTMEWAIT);  
  else if(strcmp("GOLEM", pName) == 0)
      return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        ISGREY, NEUTRAL, false, true, true, true, false, NO);                     
  else if(strcmp("FURY", pName) == 0) 
      return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        ISGREY, ENEMY, false, true, true, true, false, NO);   
  else if(strcmp("FLAME", pName) == 0) 
      return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, REVERSE, Allowed, X, Y,
                        ISGREY, ENEMY, false, true, true, true, false, NO);    
  else if(strcmp("SWITCH", pName) == 0)
  {
   if(pColour == ISYELLOW) pColour = SETYELLOW;
   if(pColour == ISRED) pColour = SETRED;     
      return MakePlayTile(pName, STOPPED, false, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, false, false, false, false, false, NO);      
  }                          
  else if(strcmp("TRAP", pName) == 0)
        return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, true, true, false, true, false, NO);        
  else if(strcmp("ARROW", pName) == 0) 
        return MakePlayTile(pName, STOPPED, pMoveable, false, true, pPointDirection, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, false, false, false, false, false, NO);                               
  else if(strcmp("HVBLOCK", pName) == 0)
        return MakePlayTile(pName, PUSH, pMoveable, true, false, PNONE, BLOCK, Allowed, X, Y,
                        ISGREY, NEUTRAL, false, false, false, true, false, NO);     
  else if(strcmp("VBLOCK", pName) == 0)
  {
   Allowed[1] = false; Allowed[3] = false;    
        return MakePlayTile(pName, PUSH, pMoveable, true, false, PNONE, BLOCK, Allowed, X, Y,
                        ISGREY, NEUTRAL, false, false, false, true, false, NO);       
  }  
  else if(strcmp("HBLOCK", pName) == 0)
  {
   Allowed[0] = false; Allowed[2] = false;       
        return MakePlayTile(pName, PUSH, pMoveable, true, false, PNONE, BLOCK, Allowed, X, Y,
                        ISGREY, NEUTRAL, false, false, false, true, false, NO);       
  }
  else if(strcmp("DBLOCK", pName) == 0)
        return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, true, false, false, true, false, NO);         
  else 
        return MakePlayTile(pName, STOPPED, pMoveable, false, false, PNONE, NORMAL, Allowed, X, Y,
                        pColour, NEUTRAL, true, false, false, true, false, NO);  
}

int DetermineNewDirection(PlayTile TileA, PlayTile TileB, ColourType Active)
//Tile A is hitting Tile B.
//returns 0 if Tile A nor B are changing directions.  returns 1 if Tile A is forcing a change on
// TileB.  2 if tile B is forcing change on TileA.  3 if Tile A is giving directions to B.  
//If a tile is forced a NEW direction, it must be the next tile to be resolved in this round.  
//If a tile is given a NEW direction, it is the next to be resolved, and this old tile is resolved
//immediately afterwards in this current round.
{
 if((TileB.Colour == ISGREY || TileB.Colour == Active) && (TileA.Colour == ISGREY || TileA.Colour == Active))
 { 
  if(TileA.ForceDirections && TileB.InTheWay && TileB.Moving != (Direction) TileA.PointDirection) return 1;   
  else if(TileB.ForceDirections && TileA.InTheWay && TileA.Moving != (Direction) TileB.PointDirection) return 2;
  else if(TileA.GivesDirections && TileB.MoveType == BLOCK  && TileB.Moving != STOPPED) return 3;
//  else if(TileB.GivesDirections && TileA.MoveType == BLOCK  && TileA.Moving != STOPPED) return 3;
 }

 return 0;
}

int SeeInteraction(PlayTile TileA, PlayTile TileB, ColourType Active)
{
 if((TileA.Moving == STOPPED || TileA.Moving == PUSH)) return 0;
 else if((TileA.Colour == ISGREY || TileA.Colour == Active || TileA.Colour > 2)
    && (TileB.Colour == ISGREY || TileB.Colour == Active  || TileB.Colour > 2))
 {
  if(TileA.Moving == NORTH) TileA.Y--;
  else if(TileA.Moving == SOUTH) TileA.Y++;
  else if(TileA.Moving == WEST) TileA.X--;
  else if(TileA.Moving == EAST) TileA.X++;  
  
  if(TileA.X == TileB.X && TileA.Y == TileB.Y)
  {
   Direction Opposite = TileA.Moving;
   Opposite = (Direction)((int)TileA.Moving + 2);
   if(Opposite > WEST) Opposite = (Direction)((int)Opposite % 4);
   if(TileB.Moving > STOPPED && TileB.Moving < PUSH && TileB.Moving != Opposite) return 2;
   else return 1;
            // printf("aX: %d aY: %d bX: %d bY: %d\r\n", TileA.X, TileA.Y, TileB.X, TileB.Y);
  }
  else return 0;
 }
 else return 0;
}

int RemoveTile(PlayTile TileA, PlayTile TileB, ColourType Active)
//Tile A is hitting Tile B.
//Returns 0 if both are still present.  1 if removing TileA.  2 if removing TileB.  3 if
//removing both.
{
 if((TileA.Colour == ISGREY || TileA.Colour == Active || TileA.Colour > 2)
    && (TileB.Colour == ISGREY || TileB.Colour == Active || TileB.Colour > 2))
    {
     if(TileB.Murderer && TileA.Liveable)
     {
//       printf("%s just murdered %s.\r\n", TileB.Name, TileA.Name);                 
//       if(TileA.Important)
//         printf("Game over.\r\n");
       if(TileB.Suicidal)
       {
//         printf("%s has also died.\r\n", TileB.Name);
         return 3;
       }
       else return 1;
     }
     else if(TileA.Murderer && TileB.Liveable)
     {
//       printf("%s just murdered %s.\r\n", TileA.Name, TileB.Name);
//       if(TileB.Important)
//         printf("Game over.\r\n");
       if(TileA.Suicidal)
       {
//         printf("%s has also died.\r\n", TileA.Name);
         return 2;
       }
       else return 1;
     }
     else if((TileA.Alignment == HERO && TileB.Alignment == ENEMY)
              || (TileB.Alignment == HERO && TileA.Alignment == ENEMY))
     {
//       printf("%s and %s have murdered eachother.\r\n", TileA.Name, TileB.Name);
       return 3;
     }
     else if(TileA.Collect == COLLECTOR && TileB.Collect == COLLECTABLE)
     {
//       printf("%s just collected %s.\r\n", TileA.Name, TileB.Name); 
       return 2;    
     }
     else if(TileB.Collect == COLLECTOR && TileA.Collect == COLLECTABLE)
     {
//       printf("%s just collected %s.\r\n", TileB.Name, TileA.Name);     
       return 1;
     }
    }
    return 0;
}

bool Takeover(PlayTile TileA, PlayTile TileB, ColourType Active)
{
 if(TileA.Colour != Active && TileA.Colour != ISGREY) TileA.InTheWay = false;    
 if(TileB.Colour != Active && TileB.Colour != ISGREY) TileB.InTheWay = false;
 if(TileA.InTheWay && TileB.InTheWay) return false;
 else return true;
}

PlayTile SetDirectionByPlayer(PlayTile theTile, Direction NewDir)
{
 if(theTile.Moveable)
 {
     if(NewDir == STOPPED)
     {
       theTile.Moving = STOPPED;
       return theTile;
     }
     if(NewDir >= PUSH)
     {
      if(theTile.MoveType == BLOCK) theTile.Moving = PUSH;
      return theTile;  
     }
     if(NewDir > STOPPED)
     {
      if(theTile.MoveType == NORMAL)
      {
       if(theTile.Allowed[NewDir-1])
       {
        theTile.Moving = NewDir;
        return theTile;
       }
      }
      else if(theTile.MoveType == REVERSE)
      {
       if(NewDir == NORTH) NewDir = SOUTH;
       else if(NewDir == EAST) NewDir = WEST;
       else if(NewDir == SOUTH) NewDir = NORTH;
       else if(NewDir == WEST) NewDir = EAST;
       
       if(theTile.Allowed[NewDir])
       {
        theTile.Moving = NewDir;
        return theTile;
       }                   
      }
     }
 }
 return theTile;
}

/*
int main()
{
 PlayTile test1 = MakeStandardTile("PID", ISGREY, true, PNONE, 0, 0);
 PlayTile test2 = MakeStandardTile("GOLEM", ISGREY, true, PNONE, 0, 0);
 int result = RemoveTile(test1, test2);
 if(result == 0) 
 {
  printf("Both tiles lived.\r\n");
  int direction = DetermineDirection(test1, test2);
  if(direction == 0) 
  {
    printf("%s is overtaking %s.\r\n", test1.Name, test2.Name);
    int position = Reposition(test1, test2);
    if(position == 0) printf("%s has stopped right where it is.\r\n", test1.Name);
    else if(position == 1) printf("%s has stopped ontop of %s.\r\n", test1.Name, test2.Name);
    else if(position == 2) printf("%s has moved beneath %s.\r\n", test1.Name, test2.Name);
  }
  else if(direction == 1) 
    printf("%s forces a new direction on %s.\r\nAdd %s to move list and finish resolving %s.\r\n", test1.Name, test2.Name, test2.Name, test1.Name);
  else if(direction == 2) 
    printf("%s forces a new direction on %s.\r\nRestart resolving %s with different movement.\r\n", test2.Name, test1.Name, test1.Name);
  else if(direction == 3) 
    printf("%s gives a new direction on %s. Abandon %s.\r\nAdd %s THEN %s to Resolution List.\r\n", test1.Name, test2.Name, test1.Name, test2.Name, test1.Name);  
 }
 else if(result == 1) printf("Removing %s\r\n", test1.Name);
 else if(result == 2) printf("Removing %s\r\n", test2.Name);
 else if(result == 3) printf("Removed both tiles.\r\n");
 getchar();
}
*/


